//=======================================================================
/**
 * @file BinomialGraph.h
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef BINOMIALGRAPH_H
#define BINOMIALGRAPH_H

#include <QChart>
#include <QChartView>

#include "SessionTrial.h"

/**
 * @ingroup StatisticsUI
 * @class BinomialGraph
 * @brief A graphical representation of a binomial and inverse binomial distribution based on session trials.
 *
 * It inherits from QChartView and overrides some of its functionality.
 * It also utilized for inverse binmomial distribution presentation of the session trials.
 */
class BinomialGraph : public QChartView
{
    Q_OBJECT

public:
    explicit BinomialGraph(QWidget *parent = nullptr);

    /**
     * @brief Update the graph with a new set of trials.
     *
     * This function updates the graph to reflect the distribution of the given trials.
     *
     * @param trials The new set of trials to represent in the graph.
     */
    void updateGraph(const QVector<SessionTrial> &trials);

private:
    /**
     * @brief Create a new binomial chart based on a set of trials.
     *
     * This function creates a new QChart object that represents the binomial distribution of the given trials.
     *
     * @param trials The trials to base the chart on.
     * @return A pointer to the newly created chart.
     */
    QChart *createBinomialChart(const QVector<SessionTrial> &trials);
};

#endif // BINOMIALGRAPH_H


