//=======================================================================
/**
 * @file binomialhelper.h
 * @author Priyonto M Rahman
 * @copyright Copyright (c) 2023, Priyonto M Rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef BINOMIALHELPER_H
#define BINOMIALHELPER_H

#include <cmath>

/**
 * @ingroup Utils
 * @class BinomialHelper
 * @brief Provides functions for binomial distribution calculations.
 */
class BinomialHelper {
public:
    /**
     * @brief Calculates the natural logarithm of the factorial of an integer.
     * @param n The integer.
     * @return The natural logarithm of the factorial of n.
     */
    static double logFactorial(int n);

    /**
     * @brief Calculates the natural logarithm of the combination of n items taken k at a time.
     * @param n The total number of items.
     * @param k The number of items to choose.
     * @return The natural logarithm of the combination of n items taken k at a time.
     */
    static double logCombination(int n, int k);

    /**
     * @brief Calculates the combination of n items taken k at a time.
     * @param n The total number of items.
     * @param k The number of items to choose.
     * @return The combination of n items taken k at a time.
     */
    static int combination(int n, int k);

    /**
     * @brief Calculates the binomial probability.
     * @param n The number of trials.
     * @param k The number of successes.
     * @param p The probability of success on a single trial.
     * @return The binomial probability of getting k successes in n trials.
     */
    static double binomialProbability(int n, int k, double p);

    /**
     * @brief Calculates the cumulative distribution function (CDF) for a binomial distribution.
     * @param n The number of trials.
     * @param k The number of successes.
     * @param p The probability of success on a single trial.
     * @return The cumulative probability of getting up to k successes in n trials.
     */
    static double binomialCDF(int n, int k, double p);

    /**
     * @brief Calculates the inverse of the binomial CDF.
     * @param targetP The target cumulative probability.
     * @param n The number of trials.
     * @param p The probability of success on a single trial.
     * @return The smallest number of successes such that the CDF is greater than or equal to targetP.
     */
    static int inverseBinomialCDF(double targetP, int n, double p);
};

#endif // BINOMIALHELPER_H
