//=======================================================================
/**
 * @file DatabaseManager.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef DATABASEMANAGER_H
#define DATABASEMANAGER_H

#include <QObject>
#include <QSqlDatabase>

#include "AudioSample.h"
#include "TestingSession.h"
#include "SessionTrial.h"
#include "TestSessionData.h"

/**
 * @ingroup DataAnalysisReporting
 * @class DatabaseManager
 * @brief The DatabaseManager class manages the database operations for the audio testing application.
 *
 * This class provides an interface to create, read, update, and delete data from the database.
 * It encapsulates all the database operations and provides methods to interact with the database.
 */
class DatabaseManager : public QObject {
    Q_OBJECT

public:
    /**
     * @brief Constructs a new DatabaseManager object.
     *
     * @param parent The parent QObject.
     */
    explicit DatabaseManager(QObject *parent = nullptr);

    /**
     * @brief Destroys the DatabaseManager object and cleans up.
     */
    ~DatabaseManager();

    /**
     * @brief Sets up the database. Opens the connection and creates necessary tables.
     *
     * @return true if the database was set up successfully, false otherwise.
     */
    bool setupDatabase();

    /**
     * @brief Drops all tables and recreates them. All data is lost.
     */
    void dropAndRecreateTables();

    /**
     * @brief Inserts an audio file record into the database.
     *
     * @param path The path of the audio file.
     * @param fileName The name of the audio file.
     * @return true if the record was inserted successfully, false otherwise.
     */
    bool insertAudioFile(const QString& path, const QString& fileName);

    /**
     * @brief Retrieves all audio files from the database.
     *
     * @return A list of AudioSample objects representing the audio files.
     */
    QList<AudioSample> getAudioFiles();

    /**
     * @brief Deletes all audio files from the database.
     *
     * @return true if the records were deleted successfully, false otherwise.
     */
    bool clearAudioFiles();

    /**
     * @brief Creates a new test session record in the database.
     *
     * @param sampleId The ID of the sample for the test session.
     * @param trialsCount The number of trials in the test session.
     * @return The ID of the created session if the operation was successful, -1 otherwise.
     */
    int createTestSession(int sampleId, int trialsCount);

    /**
     * @brief Updates the end time of a test session record in the database.
     *
     * @param sessionId The ID of the session to be updated.
     * @return true if the record was updated successfully, false otherwise.
     */
    bool updateTestSessionEndTime(int sessionId);

    /**
     * @brief Retrieves all test sessions along with their durations from the database.
     *
     * @return A vector of TestingSession objects representing the test sessions.
     */
    QVector<TestingSession> getSessionsWithDuration();

    /**
     * @brief Inserts a new trial record into the database.
     *
     * @param sessionId The ID of the session for the trial.
     * @param correctResponse The correct response for the trial.
     * @param userResponse The user's response for the trial.
     * @param responseTime The time taken by the user to respond.
     * @return true if the record was inserted successfully, false otherwise.
     */
    bool insertTrial(int sessionId, const QString& correctResponse, const QString& userResponse, int responseTime);

    /**
     * @brief Retrieves all trials for a specific session from the database.
     *
     * @param sessionId The ID of the session for which the trials are to be retrieved.
     * @return A vector of SessionTrial objects representing the trials.
     */
    QVector<SessionTrial> getTrialsForSession(int sessionId);

    /**
     * @brief Gets the average number of trials per session.
     *
     * This method calculates and returns the average number of trials conducted
     * per session in the database.
     *
     * @return The average number of trials per session as a double.
     */
    double getAverageTrialsPerSession();

    /**
     * @brief Gets the average response time per trial.
     *
     * This method calculates and returns the average response time per trial
     * from the data in the database.
     *
     * @return The average response time per trial as a double.
     */
    double getAverageResponseTimePerTrial();

    /**
     * @brief Gets the average duration per session.
     *
     * This method calculates and returns the average duration of sessions
     * from the data in the database.
     *
     * @return The average duration per session as a double.
     */
    double getAverageDurationPerSession();

    /**
     * @brief Gets the average success per session.
     *
     * This method calculates and returns the average success rate
     * per session from the data in the database.
     *
     * @return The average success rate per session as a double.
     */
    double getAverageSuccessPerSession();

    /**
     * @brief Get confusion matrix counts for a specific session.
     *
     * This function retrieves counts for all combinations of actual and predicted values ("A" or "B") for a specified session.
     * The counts represent the number of instances in the session where the actual value and predicted value pair occurred.
     *
     * The outer QMap has "actual" values ("A" or "B") as keys and the inner QMap as values.
     * The inner QMap has "predicted" values ("A" or "B") as keys and counts as values.
     *
     * @param sessionId The ID of the session for which to retrieve the confusion matrix counts.
     * @return A QMap containing the counts of each possible actual/predicted value combination for the specified session.
     */
    QMap<QString, QMap<QString, int>> getConfusionMatrixCountsForSession(int sessionId);

    /**
     * @brief Returns the export data for all test sessions.
     *
     * This function retrieves and returns a QList<TestSessionData> containing the export data for all available test sessions.
     *
     * @return A QList<TestSessionData> containing the export data for all test sessions.
     */
    QList<TestSessionData> returnExportDataForAllSessions();

    /**
     * @brief Returns the export data for a specific test session.
     *
     * This function retrieves and returns a QList<TestSessionData> containing the export data for the test session identified by the provided sessionId.
     *
     * @param sessionId The identifier of the test session to retrieve export data for.
     * @return A QList<TestSessionData> containing the export data for the specified test session.
     */
    QList<TestSessionData> returnExportDataForSession(int sessionId);


private:
    QSqlDatabase sql_db;

    /**
     * @brief Opens a connection to the database.
     *
     * @return true if the connection was opened successfully, false otherwise.
     */
    bool openDatabase();

    /**
     * @brief Closes the connection to the database.
     */
    void closeDatabase();

    /**
     * @brief Executes a SQL query on the database.
     *
     * @param query The SQL query to be executed.
     * @return true if the query was executed successfully, false otherwise.
     */
    bool executeQuery(const QString& query);

    /**
     * @brief Executes a SQL query on the database.
     *
     * @param query The QSqlQuery object representing the query to be executed.
     * @return true if the query was executed successfully, false otherwise.
     */
    bool executeQuery(QSqlQuery& query);
};

#endif // DATABASEMANAGER_H
