//=======================================================================
/**
 * @file EQGraph.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef EQGRAPH_H
#define EQGRAPH_H

#include <QChartView>
#include <QChart>
#include <QLineSeries>
#include <QValueAxis>
#include <QCategoryAxis>
#include <QVBoxLayout>
#include <QLogValueAxis>

/**
 * @ingroup TrainingUI
 * @class EQGraph
 * @brief A class to plot frequency graph.
 *
 * EQGraph is a widget that plots graphs using Qt Charts.
 * The X-axis is logarithmic, making it suitable for frequency response plots.
 */
class EQGraph : public QWidget
{
public:
    /**
     * @brief Construct a new EQGraph object.
     *
     * @param parent The parent widget.
     */
    EQGraph(QWidget *parent = nullptr);

    /**
     * @brief Destroy the EQGraph object and free any allocated resources.
     */
    ~EQGraph();

    /**
     * @brief Plot a graph.
     *
     * The data in the x and y vectors is plotted on the graph. The x-axis is logarithmic.
     *
     * @param x The x values.
     * @param y The y values.
     * @param magnitude The magnitude range.
     * @param title The title of the graph.
     * @param xAxisLabel The label for the x-axis.
     * @param yAxisLabel The label for the y-axis.
     */
    void plot(const std::vector<double>& x,
              const std::vector<double>& y,
              int magnitude,
              const QString& title,
              const QString& xAxisLabel,
              const QString& yAxisLabel);

private:
    QChart *m_chart;             ///< The chart object.
    QChartView *m_chartView;     ///< The chart view.
    QLineSeries *m_series;       ///< The series of data to be plotted.
    QLogValueAxis *m_axisX;      ///< The x-axis.
    QValueAxis *m_axisY;         ///< The y-axis.
};

#endif // EQGRAPH_H
