//=======================================================================
/**
 * @file JackWorker.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================


#ifndef JACKWORKER_H
#define JACKWORKER_H

#include <QObject>
#include <QMutex>

#include "AudioFile.h"
#include "SourceType.h"
#include "FilterType.h"

class JackClientAudioEngine;

/**
 * @ingroup JackExternalClient
 * @class JackWorker
 * @brief The JackWorker class is responsible for managing the Jack audio server and audio client engine.
 *
 * JackWorker is a QObject that handles interactions with the Jack audio server.
 * It manages an instance of JackClientAudioEngine, handles signals and slots for
 * managing the server and updating settings, and communicates server status
 * updates through the jackEngineStatus signal.
 */
class JackWorker : public QObject
{
    Q_OBJECT
public:
    /**
     * @brief Constructs a JackWorker object.
     *
     * @param parent The parent QObject.
     */
    explicit JackWorker(QObject *parent = nullptr);

    /**
     * @brief Checks if the audio engine is running.
     *
     * @return True if the audio engine is running, false otherwise.
     */
    bool isAudioEngineRunning();

signals:
    /**
     * @brief Signal emitted when the Jack server's status changes.
     *
     * @param status The new status of the Jack server.
     */
    void jackEngineStatus(QString);

    /**
     * @brief Signal emitted when the CPU load method is pinged.
     *
     * @param load Current DSP CPU load in floating point.
     */
    void updateCPULoad(float);

    /**
     * @brief Signal emitted when the Jack server is finished.
     */
    void finished();

public slots:
    /**
     * @brief Sets up the Jack server.
     */
    void setupJackServer();

    /**
     * @brief Starts the Jack server.
     */
    void startJackServer();

    /**
     * @brief Stops the Jack server.
     */
    void stopJackServer();

    /**
     * @brief Sets the audio source.
     *
     * @param type The type of the audio source.
     */
    void setAudioSource(SourceType type);

    /**
     * @brief Sets the active filter type.
     *
     * @param filterType The new active filter type.
     */
    void setActiveFilterType(FilterType filterType);

    /**
     * @brief Sets the volume level.
     *
     * @param level The new volume level.
     */
    void setVolumeLevel(int level);

    /**
     * @brief Sets the filter configuration.
     *
     * @param frequency The new filter frequency.
     * @param gain The new filter gain.
     */
    void setFilterConfig(int frequency, int gain);

    /**
     * @brief Sets the audio file.
     *
     * @param file The new audio file.
     */
    void setAudioFile(AudioFile<float> file);

    /**
     * @brief Sets whether the audio is playing modified.
     *
     * @param playingModified True if the audio should be played modified, false otherwise.
     */
    void setIsPlayingModified(bool playingModified);

    /**
     * @brief Ping for CPU Load
     *
     * @param Ping the AudioEngine for current CPU Load.
     */
    void pingCPULoad();

private:
    /**
     * @brief Resets the audio engine.
     */
    void resetAudioEngine();

    /**
     * @brief Checks if the audio engine is active.
     *
     * @return True if the audio engine is active, false otherwise.
     */
    bool isAudioEngineActive() const;

    JackClientAudioEngine* audioEngine = nullptr;
    SourceType sourceType = SourceType::Local;
    bool isRunning = false;
};

#endif // JACKWORKER_H
