//=======================================================================
/**
 * @file ShelfFilter.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================


#ifndef SHELFFILTER_H
#define SHELFFILTER_H

#include "BaseFilter.h"

#include <iostream>
#include <array>
#include <vector>
#include <cmath>
#include <complex>

#include "FilterType.h"

/**
 * @ingroup JackExternalClient
 * @class ShelfFilter
 * @brief High-shelf and low-shelf filter for audio processing
 *
 * This class implements a high-shelf and low-shelf filter based on the BaseFilter interface. It is used in audio processing to attenuate or boost frequencies above or below a certain cutoff frequency.
 */
class ShelfFilter: public BaseFilter {
public:
    /**
     * @brief Constructor
     *
     * Initializes the filter with a sample rate, cutoff frequency, gain in decibels, and type of filter.
     *
     * @param sample_rate The sample rate of the audio data in Hz
     * @param cutoff_frequency The cutoff frequency of the filter in Hz
     * @param gain_db The gain or attenuation above/below the cutoff frequency, in decibels
     * @param type The type of filter: HighShelf or LowShelf
     */
    ShelfFilter(double sample_rate, double cutoff_frequency, double gain_db, FilterType type);

    /**
     * @brief Processes a buffer of input samples through the filter. This is experimental and not used.
     *
     * This method takes a vector of floating point samples as input,
     * processes each sample through the filter, and returns a new vector
     * containing the filtered samples.
     *
     * This method is meant to be used when it's more efficient to process
     * a whole buffer of samples at once, rather than calling process_sample()
     * on each individual sample. Experimental and currently not utilized.
     *
     * @param inputBuffer A vector of input samples to be processed by the filter.
     * @return A vector of output samples that have been processed by the filter.
     */
    std::vector<float> process_buffer(std::vector<float>& inputBuffer) override;

    /**
     * @brief Override of process_sample for ShelfFilter class.
     *
     * This function is a specific implementation of process_sample for the ShelfFilter.
     * It processes a single audio sample through the shelf filter, and provides the
     * resulting output.
     *
     * @param sample The sample to be processed. This is a floating point value
     *               representing the audio sample.
     * @return The processed sample. The processing applies the specific shelf filter
     *         calculations to the input sample.
     */
    float process_sample(float sample) override;

    /**
     * @brief Get the A coefficients
     *
     * Returns the current A (denominator) coefficients of the filter.
     *
     * @return The A coefficients
     */
    const std::vector<double>& get_A() const;

    /**
     * @brief Get the B coefficients
     *
     * Returns the current B (numerator) coefficients of the filter.
     *
     * @return The B coefficients
     */
    const std::vector<double>& get_B() const;

private:
    std::array<double, 3> x = {0, 0, 0}; ///< Input samples buffer
    std::array<double, 3> y = {0, 0, 0}; ///< Output samples buffer
    std::vector<double> A; ///< Denominator coefficients
    std::vector<double> B; ///< Numerator coefficients
    FilterType filterType; ///< Type of the filter: HighShelf or LowShelf

    /**
     * @brief Update filter coefficients
     *
     * Recalculates the filter coefficients based on the current sample rate, cutoff frequency, gain, and filter type. This method is called internally whenever these parameters change.
     *
     * @param sample_rate The sample rate of the audio data in Hz
     * @param cutoff_frequency The cutoff frequency of the filter in Hz
     * @param gain_db The gain or attenuation above/below the cutoff frequency, in decibels
     */
    void updateCoefficients(double sample_rate, double cutoff_frequency, double gain_db);
};

#endif // SHELFFILTER_H
