//=======================================================================
/**
 * @file StatisticsWindow.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef STATISTICSWINDOW_H
#define STATISTICSWINDOW_H

#include <QWidget>
#include <QChart>

#include "DatabaseManager.h"
#include "SessionListWidget.h"
#include "StatisticsSummaryWidget.h"
#include "BinomialGraph.h"
#include "inversebinomialgraph.h"
#include "ConfusionMatrixWidget.h"

/**
 * @ingroup StatisticsUI
 * @class StatisticsWindow
 * @brief Provides a widget for displaying session statistics.
 *
 * This class is responsible for managing and displaying the statistics of different sessions.
 * It has methods for creating widgets, populating sessions, creating graphs and updating graphs.
 */
class StatisticsWindow : public QWidget
{
    Q_OBJECT
public:
    explicit StatisticsWindow(QWidget *parent = nullptr); ///< Constructs a StatisticsWindow with an optional parent widget.
    ~StatisticsWindow(); ///< Destructor.

    QWidget* createWidget(); ///< Creates the widget for the statistics window.

    void setDatabaseManager(DatabaseManager* manager); ///< Sets the database manager.
    void populateSessions(); ///< Populates the list of sessions.

    QChart* createBinomialGraph(QVector<SessionTrial> &trials); ///< Creates a binomial graph from the given session trials.

private:
    DatabaseManager* dbManager; ///< The database manager.

    StatisticsSummaryWidget* summaryWidget; ///< The statistics summary widget.

    SessionListWidget* sessionListWidget; ///< The session list widget.
    QWidget* sessionListContentWidget; ///< The content widget for the session list.

    BinomialGraph *binomialGraph; ///< The binomial graph.
    InverseBinomialGraph *inverseBinomialGraph; ///< The inverse binomial graph.

    ConfusionMatrixWidget *confusionMatrixWidget; ///< The widget for confusion matrix.

    int currenSessionId; ///< Session selected currently from the list.

private slots:
    void updateBinomialGraphWithSession(int sessionId); ///< Updates the binomial graph with the given session ID.
    void exportAllSessions();
    void exportSelectedSession(int sessionId);
};

#endif // STATISTICSWINDOW_H
