//=======================================================================
/**
 * @file TestingWindow.h
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#ifndef TESTINGWINDOW_H
#define TESTINGWINDOW_H

#include <QElapsedTimer>
#include <QLabel>
#include <QPushButton>
#include <QSpinBox>
#include <QWidget>

#include "AudioFile.h"
#include "BaseFilter.h"
#include "CustomComboBox.h"
#include "DatabaseManager.h"
#include "FilterType.h"
#include "JackWorker.h"
#include "SessionType.h"
#include "TrackType.h"

/**
 * @ingroup TestingUI
 * @class TestingWindow
 * @brief Provides a widget for conducting testing sessions.
 *
 * This class is responsible for managing and displaying the testing window.
 * It includes methods for creating widgets, populating audio files, updating the selected
 * audio file and filter parameters, and managing the testing session state.
 */
class TestingWindow : public QWidget
{
    Q_OBJECT
public:
    explicit TestingWindow(QWidget *parent = nullptr); ///< Constructs a TestingWindow with an optional parent widget.
    ~TestingWindow(); ///< Destructor.

    QWidget* createWidget(); ///< Creates the widget for the testing window.

    void setDatabaseManager(DatabaseManager* manager); ///< Sets the database manager.
    void populateAudioFiles(); ///< Populates the list of audio files.

    void updateSelectedAudioFile(QString filePath); ///< Updates the selected audio file.
    void updateSelectedFilterType(FilterType filterType); ///< Updates the selected filter type.
    void updateFilterParameters(int gain, int cutOffFrequency); ///< Updates filter parameters.

signals:
    void didChangeTestingSessionState(SessionType sessionType); ///< Signal emitted when the testing session state changes.

private:
    QPushButton* startStopButton;
    QSpinBox* numberOfTrialsSpinbox;
    CustomComboBox *trackSelectionDropDown;

    QLabel* attemptCountLabel;
    QLabel* currentlyPlayinglabel;

    // Database
    DatabaseManager* dbManager;

    // Threading
    QThread* workerThread;
    JackWorker* jackWorker;

    QList<AudioSample> audioFiles;

    AudioFile<float> currentAudioFileData;
    QString selectedAudioFilePath;

    int selectedAudioFileId;

    FilterType activeFilterType = FilterType::HighShelf;
    std::shared_ptr<BaseFilter> activeFilter;

    TrackType buttonATrackType;
    TrackType buttonBTrackType;

    int maxTrialsCount = 5;
    int currentTrialCursor = 0;

    int gain = 6;
    int cutOffFrequency = 8000;

    int currentSession = -1;

    QElapsedTimer responseTimer;

private slots:
    void shuffleTracks(TrackType &buttonATrackType, TrackType &buttonBTrackType); ///< Shuffles the tracks.
    void didSelectAudioFile(AudioSample audioSample); ///< Handles the event of an audio file selection.
    void startPlayingTrackA(); ///< Starts playing track A.

    void showErrorDialog(const QString &message); ///< Shows an error dialog.
    void showSessionStartConfirmationDialog(const QString &message); ///< Shows a session start confirmation dialog.
    void showSessionStopConfirmationDialog(const QString &message); ///< Shows a session stop confirmation dialog.
    void showSessionEarlyEndConfirmationDialog(const QString &message); ///< Shows a session early end confirmation dialog.
    void startTestingSessionAction(); ///< Starts the testing session.
    void stopTestingSessionAction(); ///< Stops the testing session.
    void startResponseTimer(); /// Starts the response time tracking timer.
    void restartResponseTimer(); /// <Restarts the response time tracking timer.

    bool loadSelectedAudioFile(QString filePath); ///< Loads the selected audio file.
};


#endif // TESTINGWINDOW_H

