//=======================================================================
/**
 * @file DatabaseManager.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "DatabaseManager.h"

#include <QSqlQuery>
#include <QSqlError>
#include <QDateTime>
#include <QDebug>

DatabaseManager::DatabaseManager(QObject *parent) : QObject(parent) {
    sql_db = QSqlDatabase::addDatabase("QSQLITE");
    sql_db.setDatabaseName("audio_testing.db");
}

DatabaseManager::~DatabaseManager() {
    closeDatabase();
}

bool DatabaseManager::openDatabase() {
    if (!sql_db.open()) {
        qDebug() << "Error opening database: " << sql_db.lastError().text();
        return false;
    }
    return true;
}

void DatabaseManager::closeDatabase() {
    if (sql_db.isOpen()) {
        sql_db.close();
    }
}

bool DatabaseManager::executeQuery(const QString& queryStr) {
    QSqlQuery query;
    bool result = query.exec(queryStr);
    if (!result) {
        qDebug() << "Database error: " << query.lastError();
    }
    return result;
}

bool DatabaseManager::executeQuery(QSqlQuery& query) {
    bool result = query.exec();
    if (!result) {
        qDebug() << "Database error: " << query.lastError();
    }
    return result;
}

bool DatabaseManager::setupDatabase() {
    if (!openDatabase()) {
        qDebug() << "Error: Unable to open database.";
        return false;
    }

    // setup database schema
    QString createAudioSamplesTableQuery = R"(
        CREATE TABLE IF NOT EXISTS audio_samples (
            sample_id INTEGER PRIMARY KEY,
            file_name VARCHAR(255),
            path VARCHAR(255),
            other_properties VARCHAR(255)
        )
    )";
    QString createTestSessionsTableQuery = R"(
        CREATE TABLE IF NOT EXISTS test_sessions (
            session_id INTEGER PRIMARY KEY,
            sample_id INTEGER,
            max_trials INTEGER,
            start_time TIMESTAMP,
            end_time TIMESTAMP,
            FOREIGN KEY (sample_id) REFERENCES audio_samples(sample_id)
        )
    )";
    QString createTrialsTableQuery = R"(
        CREATE TABLE IF NOT EXISTS trials (
            trial_id INTEGER PRIMARY KEY,
            session_id INTEGER,
            correct_response VARCHAR(255),
            user_response VARCHAR(255),
            response_time INTEGER,
            FOREIGN KEY (session_id) REFERENCES test_sessions(session_id)
        )
    )";

    return executeQuery(createAudioSamplesTableQuery) &&
           executeQuery(createTestSessionsTableQuery) &&
           executeQuery(createTrialsTableQuery);
}

void DatabaseManager::dropAndRecreateTables() {
    if (!openDatabase()) {
        return;
    }

    executeQuery("DROP TABLE IF EXISTS audio_samples;");
    executeQuery("DROP TABLE IF EXISTS test_sessions;");
    executeQuery("DROP TABLE IF EXISTS trials;");

    setupDatabase();
}


bool DatabaseManager::insertAudioFile(const QString& path, const QString& fileName) {
    if (!openDatabase()) {
        return false;
    }

    QSqlQuery query;
    query.prepare("SELECT * FROM audio_samples WHERE path = ?");
    query.addBindValue(path);

    if (!executeQuery(query)) {
        return false;
    }

    if (query.next()) {
        return false;
    }

    query.prepare("INSERT INTO audio_samples (file_name, path) VALUES (?, ?)");
    query.addBindValue(fileName);
    query.addBindValue(path);

    return executeQuery(query);
}

QList<AudioSample> DatabaseManager::getAudioFiles() {
    QList<AudioSample> audioSamples;

    if (!openDatabase()) {
        return audioSamples;
    }

    QSqlQuery query("SELECT sample_id, file_name, path FROM audio_samples");

    if (!executeQuery(query)) {
        return audioSamples;
    }

    while (query.next()) {
        int sampleId = query.value(0).toInt();
        QString fileName = query.value(1).toString();
        QString path = query.value(2).toString();
        audioSamples.append(AudioSample(sampleId, fileName, path));
    }

    return audioSamples;
}

bool DatabaseManager::clearAudioFiles() {
    if (!openDatabase()) {
        return false;
    }

    return executeQuery("DELETE FROM audio_samples");
}

int DatabaseManager::createTestSession(int sampleId, int trialsCount) {
    if (!openDatabase()) {
        return -1;
    }

    QSqlQuery query;
    query.prepare("INSERT INTO test_sessions (sample_id, max_trials, start_time) VALUES (?, ?, ?)");
    query.addBindValue(sampleId);
    query.addBindValue(trialsCount);
    query.addBindValue(QDateTime::currentDateTime());

    if (!executeQuery(query)) {
        return -1;
    }

    return query.lastInsertId().toInt();
}

bool DatabaseManager::updateTestSessionEndTime(int sessionId) {
    if (!openDatabase()) {
        return false;
    }

    QSqlQuery query;
    query.prepare("UPDATE test_sessions SET end_time = ? WHERE session_id = ?");
    query.addBindValue(QDateTime::currentDateTime());
    query.addBindValue(sessionId);

    return executeQuery(query);
}

QVector<TestingSession> DatabaseManager::getSessionsWithDuration() {
    QVector<TestingSession> sessions;

    if (!openDatabase()) {
        return sessions;
    }

    QSqlQuery query;
    query.prepare("SELECT session_id, sample_id, max_trials, strftime('%s', end_time) - strftime('%s', start_time) AS duration FROM test_sessions");

    if (!executeQuery(query)) {
        return sessions;
    }

    while (query.next()) {
        TestingSession sessionInfo;
        sessionInfo.sessionId = query.value(0).toInt();
        sessionInfo.sampleId = query.value(1).toInt();
        sessionInfo.trialsCount = query.value(2).toInt();
        sessionInfo.duration = query.value(3).toInt();
        sessions.append(sessionInfo);
    }

    return sessions;
}

bool DatabaseManager::insertTrial(int sessionId, const QString& correctResponse, const QString& userResponse, int responseTime) {
    if (!openDatabase()) {
        return false;
    }

    QSqlQuery query;
    query.prepare("INSERT INTO trials (session_id, correct_response, user_response, response_time) VALUES (?, ?, ?, ?)");
    query.addBindValue(sessionId);
    query.addBindValue(correctResponse);
    query.addBindValue(userResponse);
    query.addBindValue(responseTime);

    return executeQuery(query);
}

QVector<SessionTrial> DatabaseManager::getTrialsForSession(int sessionId) {
    QVector<SessionTrial> trials;

    if (!openDatabase()) {
        return trials;
    }

    QSqlQuery query;
    query.prepare("SELECT trial_id, session_id, correct_response, user_response, response_time FROM trials WHERE session_id = ?");
    query.addBindValue(sessionId);

    if (!executeQuery(query)) {
        return trials;
    }

    while (query.next()) {
        SessionTrial trial;
        trial.trialId = query.value(0).toInt();
        trial.sessionId = query.value(1).toInt();
        trial.correctResponse = query.value(2).toString();
        trial.userResponse = query.value(3).toString();
        trial.responseTime = query.value(4).toInt();
        trials.append(trial);
    }

    return trials;
}

double DatabaseManager::getAverageTrialsPerSession() {
    QVector<TestingSession> sessions = getSessionsWithDuration();
    if (sessions.isEmpty()) {
        return 0.0;
    }

    int totalTrials = 0;
    for (const TestingSession &session : sessions) {
        totalTrials += session.trialsCount;
    }

    return static_cast<double>(totalTrials) / sessions.size();
}

double DatabaseManager::getAverageResponseTimePerTrial() {
    QVector<TestingSession> sessions = getSessionsWithDuration();
    if (sessions.isEmpty()) {
        return 0.0;
    }

    qint64 totalResponseTime = 0;
    int totalTrials = 0;

    for (const TestingSession &session : sessions) {
        QVector<SessionTrial> trials = getTrialsForSession(session.sessionId);
        for (const SessionTrial &trial : trials) {
            totalResponseTime += trial.responseTime / 1000; // Response time is logged as ms.
        }
        totalTrials += trials.size();
    }

    return totalTrials > 0 ? static_cast<double>(totalResponseTime) / totalTrials : 0.0;
}

double DatabaseManager::getAverageDurationPerSession() {
    QVector<TestingSession> sessions = getSessionsWithDuration();
    if (sessions.isEmpty()) {
        return 0.0;
    }

    qint64 totalDuration = 0;
    for (const TestingSession &session : sessions) {
        totalDuration += session.duration;
    }

    return static_cast<double>(totalDuration) / sessions.size();
}

double DatabaseManager::getAverageSuccessPerSession() {
    QVector<TestingSession> sessions = getSessionsWithDuration();
    if (sessions.isEmpty()) {
        return 0.0;
    }

    int totalCorrectResponses = 0;
    int totalTrials = 0;

    for (const TestingSession &session : sessions) {
        QVector<SessionTrial> trials = getTrialsForSession(session.sessionId);
        for (const SessionTrial &trial : trials) {
            if (trial.correctResponse == trial.userResponse) {
                totalCorrectResponses++;
            }
        }
        totalTrials += trials.size();
    }

    return totalTrials > 0 ? static_cast<double>(totalCorrectResponses) / totalTrials * 100 : 0.0;
}

QMap<QString, QMap<QString, int>> DatabaseManager::getConfusionMatrixCountsForSession(int sessionId) {
    QSqlQuery query;
    const QStringList responseTypes = {"A", "B"};
    QMap<QString, QMap<QString, int>> confusionMatrixCounts;

    for (const QString &actual : responseTypes) {
        for (const QString &predicted : responseTypes) {
            QString queryString = "SELECT COUNT(*) FROM trials WHERE correct_response = :actual AND user_response = :predicted AND session_id = :sessionId";
            query.prepare(queryString);
            query.bindValue(":actual", actual);
            query.bindValue(":predicted", predicted);
            query.bindValue(":sessionId", sessionId);

            if (!query.exec()) {
                qWarning() << "Failed to execute query:" << query.lastError();
                continue;
            }

            if (query.next()) {
                int count = query.value(0).toInt();
                confusionMatrixCounts[actual][predicted] = count;
            }
        }
    }

    return confusionMatrixCounts;
}

QList<TestSessionData> DatabaseManager::returnExportDataForAllSessions()
{
    QSqlQuery query;
    query.prepare("SELECT test_sessions.session_id, "
                  "test_sessions.start_time, "
                  "test_sessions.end_time, "
                  "audio_samples.file_name, "
                  "(test_sessions.end_time - test_sessions.start_time) as duration, "
                  "trials.trial_id, "
                  "trials.correct_response, "
                  "trials.user_response, "
                  "trials.response_time "
                  "FROM test_sessions "
                  "INNER JOIN trials ON test_sessions.session_id = trials.session_id "
                  "INNER JOIN audio_samples ON test_sessions.sample_id = audio_samples.sample_id");
    query.exec();

    QList<TestSessionData> result;
    while (query.next()) {
        TestSessionData data;
        data.sessionId = query.value(0).toInt();
        data.sessionStartDate = query.value(1).toString();
        data.sessionEndDate = query.value(2).toString();
        data.sampleFileName = query.value(3).toString();
        data.sessionDuration = query.value(4).toInt();
        data.trialId = query.value(5).toInt();
        data.correctResponse = query.value(6).toString();
        data.userResponse = query.value(7).toString();
        data.responseTime = query.value(8).toInt();
        result.append(data);

    }
    return result;
}

QList<TestSessionData> DatabaseManager::returnExportDataForSession(int sessionId)
{
    QSqlQuery query;
    query.prepare("SELECT test_sessions.session_id, "
                  "test_sessions.start_time, "
                  "test_sessions.end_time, "
                  "audio_samples.file_name, "
                  "(test_sessions.end_time - test_sessions.start_time) as duration, "
                  "trials.trial_id, "
                  "trials.correct_response, "
                  "trials.user_response, "
                  "trials.response_time "
                  "FROM test_sessions "
                  "INNER JOIN trials ON test_sessions.session_id = trials.session_id "
                  "INNER JOIN audio_samples ON test_sessions.sample_id = audio_samples.sample_id "
                  "WHERE test_sessions.session_id = :sessionId");
    query.bindValue(":sessionId", sessionId);
    query.exec();

    QList<TestSessionData> result;
    while (query.next()) {
        TestSessionData data;
        data.sessionId = query.value(0).toInt();
        data.sessionStartDate = query.value(1).toString();
        data.sessionEndDate = query.value(2).toString();
        data.sampleFileName = query.value(3).toString();
        data.sessionDuration = query.value(4).toInt();
        data.trialId = query.value(5).toInt();
        data.correctResponse = query.value(6).toString();
        data.userResponse = query.value(7).toString();
        data.responseTime = query.value(8).toInt();
        result.append(data);
    }
    return result;
}

