//=======================================================================
/**
 * @file FrequencyResponseHelper.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "FrequencyResponseHelper.h"

#include <cmath>
#include <iostream>

#define _USE_MATH_DEFINES

// Function to create a logarithmically spaced vector
std::vector<double> FrequencyResponseHelper::logspace(double start, double end, int num_points) {
    std::vector<double> result;

    // Convert start and end to logarithmic scale
    double start_log10 = std::log10(start);
    double end_log10 = std::log10(end);

    // Compute the step size in the logarithmic scale
    double step_log10 = (end_log10 - start_log10) / (num_points - 1);
    for (int i = 0; i < num_points; ++i) {
        // Compute the frequency in the logarithmic scale and convert it back to linear scale
        double frequnecy = std::pow(10, start_log10 + i * step_log10);
        if (frequnecy < start)
            continue;
        result.push_back(frequnecy);
    }
    return result;
}

// Function to calculate the frequency response of a filter
std::vector<double> FrequencyResponseHelper::calculate_frequency_response(
    const std::vector<double> &B,
    const std::vector<double> &A,
    double sample_rate,
    const std::vector<double> &frequencies
    ) {
    std::vector<double> gain_db;
    // Calculate the sampling period
    double T = 1.0 / sample_rate;
    // Define a complex number j
    std::complex<double> j(0, 1);

    for (double f : frequencies) {
        // Calculate the complex exponentials for the current frequency
        std::complex<double> e_neg_j2pi_fT = std::exp(-j * 2.0 * M_PI * f * T);
        std::complex<double> e_neg_j4pi_fT = std::exp(-j * 4.0 * M_PI * f * T);

        // Compute the frequency response H(f) for the current frequency
        std::complex<double> H_f = (B[0] + B[1] * e_neg_j2pi_fT + B[2] * e_neg_j4pi_fT) / (A[0] + A[1] * e_neg_j2pi_fT + A[2] * e_neg_j4pi_fT);
        // Calculate the magnitude of H(f)
        double magnitude = std::abs(H_f);
        // Convert the magnitude to decibels
        double gain = 20 * std::log10(magnitude);
        gain_db.push_back(gain);
    }

    return gain_db;
}
