//=======================================================================
/**
 * @file JackWorker.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "JackWorker.h"

#include "JackClientAudioEngine.h"

#include <QThread>
#include <QDebug>

JackWorker::JackWorker(QObject *parent)
    : QObject{parent}
{
    resetAudioEngine();
}

bool JackWorker::isAudioEngineRunning()
{
    return isRunning;
}

void JackWorker::setupJackServer() {
    QThread::currentThread()->setObjectName("Worker Thread");
    emit jackEngineStatus("Ready");
}

void JackWorker::startJackServer()
{
    if (!isAudioEngineActive()) {
        qDebug() << "Worker::Start get called from: "<<QThread::currentThread()->objectName();
        resetAudioEngine();
        audioEngine = new JackClientAudioEngine;
        audioEngine->start();

        unsigned int outPorts = audioEngine->numPhysicalDestinationPorts();
        if (outPorts >= 2) {
            //connect our ports to physical ports
            audioEngine->connectToPhysical(0,0);
            audioEngine->connectToPhysical(1,1);
        }

        unsigned int inputPorts = audioEngine->numPhysicalSourcePorts();
        // We would assume the system have stereo input channels, if not we will not connect to the input channels.
        if (sourceType == SourceType::JackInput && inputPorts >= 2) {
            audioEngine->connectFromPhysical(0, 0);
            audioEngine->connectFromPhysical(1, 1);
        }

        isRunning = true;

        emit jackEngineStatus("Running");
    }
}

void JackWorker::stopJackServer()
{
    if (isAudioEngineActive()) {
        qDebug() << "Worker::Stop get called from: "<<QThread::currentThread()->objectName();
        audioEngine->close();
        resetAudioEngine();
        emit jackEngineStatus("Stopped");
    }
}

void JackWorker::setAudioSource(SourceType type)
{
    sourceType = type;
    if (audioEngine) {
        audioEngine->setAudioSource(type);
    }
}

void JackWorker::setActiveFilterType(FilterType filterType)
{
    if (audioEngine) {
        audioEngine->setActiveFilterType(filterType);
    }
}

void JackWorker::setVolumeLevel(int level)
{
    if (isAudioEngineActive()) {
        audioEngine->setVolumeLevel(level);
    }
}

void JackWorker::setFilterConfig(int frequency, int gain)
{
    if (isAudioEngineActive()) {
        audioEngine->setFilterConfig(frequency, gain);
    }
}

void JackWorker::setAudioFile(AudioFile<float> file)
{
    if (audioEngine) {
        audioEngine->setAudioFile(file);
    }
}

void JackWorker::setIsPlayingModified(bool playingModified)
{
    if (isAudioEngineActive()) {
        audioEngine->setIsPlayingModified(playingModified);
    }
}

void JackWorker::pingCPULoad()
{
    if (isAudioEngineActive()) {
        qDebug() << "Sample Rate: " << audioEngine->getSampleRate();
        qDebug() << "Buffer Size: " << audioEngine->getBufferSize();
        emit updateCPULoad(audioEngine->getCpuLoad());
    }
}

void JackWorker::resetAudioEngine()
{
    delete audioEngine;
    audioEngine = nullptr;
    isRunning = false;
}

bool JackWorker::isAudioEngineActive() const
{
    return audioEngine && audioEngine->getState() == JackCpp::AudioIO::active;
}
