//=======================================================================
/**
 * @file ShelfFilter.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "ShelfFilter.h"

ShelfFilter::ShelfFilter(double sample_rate, double cutoff_frequency, double gain_db, FilterType type)
    : filterType(type) {
    updateCoefficients(sample_rate, cutoff_frequency, gain_db);
}

void ShelfFilter::updateCoefficients(double sample_rate, double cutoff_frequency, double gain_db) {

    double G = std::pow(10, gain_db / 20);
    double Omega = std::tan(cutoff_frequency / sample_rate / 2);
    double sqrtG = std::sqrt(G);
    double sqrt4OmegaG = std::sqrt(2) * Omega * std::sqrt(sqrtG);
    double b0, b1, b2, a0, a1, a2;

    // Ref: Välimäki, Vesa & Reiss, Joshua. (2016). All About Audio Equalization: Solutions and Frontiers.
    // Applied Sciences. 6. 129. 10.3390/app6050129.
    if (filterType == FilterType::LowShelf) {
        b0 = (sqrtG * Omega * Omega + sqrt4OmegaG + 1)*sqrtG;
        b1 = (2 * (sqrtG * Omega * Omega - 1))*sqrtG;
        b2 = (sqrtG * Omega * Omega - sqrt4OmegaG + 1)*sqrtG;
        a0 = Omega * Omega + sqrt4OmegaG + sqrtG;
        a1 = 2 * (Omega * Omega - sqrtG);
        a2 = Omega * Omega - sqrt4OmegaG + sqrtG;
    } else {
        b0 = (sqrtG + sqrt4OmegaG + Omega * Omega)*sqrtG;
        b1 = (-2 * (sqrtG - Omega * Omega))*sqrtG;
        b2 = (sqrtG - sqrt4OmegaG + Omega * Omega)*sqrtG;
        a0 = sqrtG * Omega * Omega + sqrt4OmegaG + 1;
        a1 = 2 * (sqrtG * Omega * Omega - 1);
        a2 = sqrtG * Omega * Omega - sqrt4OmegaG + 1;
    }

    // Normalize the coefficients
    B.clear();
    A.clear();
    B.emplace_back(b0 / a0);
    B.emplace_back(b1 / a0);
    B.emplace_back(b2 / a0);
    A.emplace_back(1.0);
    A.emplace_back(a1 / a0);
    A.emplace_back(a2 / a0);
}

float ShelfFilter::process_sample(float sample) {
    // Shift old samples
    x[2] = x[1];
    x[1] = x[0];
    y[2] = y[1];
    y[1] = y[0];

    // Current sample becomes the new input
    x[0] = sample;

    // Apply filter equation to calculate the current output sample
    y[0] = B[0] * x[0] + B[1] * x[1] + B[2] * x[2] - A[1] * y[1] - A[2] * y[2];

    return y[0];
}

std::vector<float> ShelfFilter::process_buffer(std::vector<float>& inputBuffer)
{
    std::vector<float> outputBuffer(inputBuffer.size());

    // Shift old samples
    x[2] = x[1];
    x[1] = x[0];
    y[2] = y[1];
    y[1] = y[0];

    for (size_t n = 0; n < inputBuffer.size(); ++n) {
        // Shift old samples
        x[2] = x[1];
        x[1] = x[0];
        y[2] = y[1];
        y[1] = y[0];

        // Current sample becomes the new input
        x[0] = inputBuffer[n];

        // Apply filter equation to calculate the current output sample
        y[0] = B[0] * x[0] + B[1] * x[1] + B[2] * x[2] - A[1] * y[1] - A[2] * y[2];

        // Save the output sample in the output buffer
        outputBuffer[n] = y[0];
    }

    return outputBuffer;
}

const std::vector<double>& ShelfFilter::get_A() const {
    return A;
}


const std::vector<double>& ShelfFilter::get_B() const {
    return B;
}
