//=======================================================================
/**
 * @file StatisticsWindow.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "StatisticsWindow.h"

#include <QScrollArea>
#include <QVBoxLayout>
#include <QLabel>
#include <QLineSeries>
#include <QChartView>
#include <QValueAxis>
#include <QPushButton>
#include <QFileDialog>

#include "CSVExporter.h"

StatisticsWindow::StatisticsWindow(QWidget *parent) : QWidget(parent)
{
}

StatisticsWindow::~StatisticsWindow()
{
    // No op yet.
}

QWidget *StatisticsWindow::createWidget()
{
    QWidget* statisticsWidget = new QWidget();

    // Summary
    summaryWidget = new StatisticsSummaryWidget();

    QVBoxLayout* summaryLayout = new QVBoxLayout;
    summaryLayout->addWidget(summaryWidget);
    summaryLayout->addSpacing(20);

    QHBoxLayout* summaryContainerLayout = new QHBoxLayout;
    summaryContainerLayout->addStretch();
    summaryContainerLayout->addLayout(summaryLayout);
    summaryContainerLayout->addStretch();

    // Export buttons
    QVBoxLayout *exportButtonsLayout = new QVBoxLayout;
    QPushButton* exportOneButton = new QPushButton("Export Selected Session Data");
    exportOneButton->setStyleSheet("color: #FFFFFF; padding: 4px");
    exportOneButton->setFixedWidth(250);
    connect(exportOneButton, &QPushButton::clicked, [this]() {
        exportSelectedSession(currenSessionId);
    });

    QPushButton* exportAllButton = new QPushButton("Export All Session Data");
    exportAllButton->setStyleSheet("color: #FFFFFF; padding: 4px");
    exportAllButton->setFixedWidth(250);
    connect(exportAllButton, &QPushButton::clicked, [this]() {
        exportAllSessions();
    });
    exportButtonsLayout->addWidget(exportOneButton);
    exportButtonsLayout->addSpacing(10);
    exportButtonsLayout->addWidget(exportAllButton);

    // Session List Widget
    sessionListWidget = new SessionListWidget;
    sessionListContentWidget = sessionListWidget->createWidget();
    QVBoxLayout* sessionListLayout = new QVBoxLayout;
    sessionListLayout->addSpacing(10);
    sessionListLayout->addWidget(sessionListContentWidget);
    sessionListLayout->addSpacing(40);
    sessionListLayout->addLayout(exportButtonsLayout);
    sessionListLayout->addStretch();

    connect(sessionListWidget, &SessionListWidget::didSelectTestingSession, this, [this](int sessionId) {
        currenSessionId = sessionId;
        updateBinomialGraphWithSession(sessionId);
    });

    // Binomial Graph
    binomialGraph = new BinomialGraph;

    // Inverse Binomial Graph
    inverseBinomialGraph = new InverseBinomialGraph;

    QVBoxLayout* binomialGraphLayout = new QVBoxLayout;
    binomialGraphLayout->addWidget(binomialGraph);
    binomialGraphLayout->addSpacing(20);
    binomialGraphLayout->addWidget(inverseBinomialGraph);
    binomialGraphLayout->addStretch();

    binomialGraph->setFixedSize(650, 300);
    inverseBinomialGraph->setFixedSize(650, 300);

    // Confusion matrix
    QLabel *confusionMatrixLabel = new QLabel("Confusion Matrix of the Session");
    confusionMatrixLabel->setStyleSheet("color: white; font-size: 12pt");
    confusionMatrixWidget = new ConfusionMatrixWidget();

    QVBoxLayout* confusionMatrixLayout = new QVBoxLayout;
    confusionMatrixLayout->addWidget(confusionMatrixLabel);
    confusionMatrixLayout->addSpacing(20);
    confusionMatrixLayout->addWidget(confusionMatrixWidget);
    confusionMatrixLayout->addStretch();
    confusionMatrixWidget->setFixedSize(600, 450);

    // Sessions container
    QHBoxLayout* sessionsLayout = new QHBoxLayout();
    sessionsLayout->addLayout(sessionListLayout);
    sessionsLayout->addSpacing(40);
    sessionsLayout->addLayout(binomialGraphLayout);
    sessionsLayout->addSpacing(40);
    sessionsLayout->addLayout(confusionMatrixLayout);
    sessionsLayout->addStretch();

    // Vertically center the button layout and slider layout in the content widget
    QVBoxLayout* mainLayout = new QVBoxLayout(statisticsWidget);
    mainLayout->addLayout(summaryContainerLayout);
    summaryLayout->addSpacing(40);
    mainLayout->addLayout(sessionsLayout);
    mainLayout->addSpacing(40);

    return statisticsWidget;
}

void StatisticsWindow::setDatabaseManager(DatabaseManager *manager)
{
    this->dbManager = manager;
}

void StatisticsWindow::populateSessions()
{
    if (dbManager != nullptr) {
        // Summary
        summaryWidget->updateAverageDuration(dbManager->getAverageDurationPerSession());
        summaryWidget->updateAverageResponseTime(dbManager->getAverageResponseTimePerTrial());
        summaryWidget->updateAverageSuccess(dbManager->getAverageSuccessPerSession());
        summaryWidget->updateAverageTrials(dbManager->getAverageTrialsPerSession());

        // Session list
        QVector<TestingSession> sessions = dbManager->getSessionsWithDuration();
        sessionListWidget->populateList(sessions);

        // First session
        if (sessions.size() > 0) {
            currenSessionId = sessions[0].sessionId;
            updateBinomialGraphWithSession(sessions[0].sessionId);
        }
    }
}

void StatisticsWindow::updateBinomialGraphWithSession(int sessionId)
{
    QVector<SessionTrial> trials = dbManager->getTrialsForSession(sessionId);
    binomialGraph->updateGraph(trials);

    inverseBinomialGraph->updateGraph(trials);

    // Confusion matrix.
    QStringList responses = {"A", "B", "X"};
    for (const QString &actual : responses)
    {
        for (const QString &predicted : responses)
        {
            QMap<QString, QMap<QString, int>> matrix = dbManager->getConfusionMatrixCountsForSession(sessionId);
            confusionMatrixWidget->populateMatrix(matrix);
        }
    }
}

void StatisticsWindow::exportAllSessions()
{
    if (!dbManager)
        return;

    QList<TestSessionData> data = dbManager->returnExportDataForAllSessions();
//    if (data.isEmpty()) {
//        return; // Show error - TODO
//    }

    QString dir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),
                                                    "/home",
                                                    QFileDialog::ShowDirsOnly
                                                        | QFileDialog::DontResolveSymlinks);

    if (!dir.isEmpty()) {
        CSVExporter exporter;
        QString filePath = QFileDialog::getSaveFileName(this, tr("Save CSV"),
                                                        "/home/username/TestingSessions.csv",
                                                        tr("CSV Files (*.csv)"));
        exporter.exportToCSV(data, filePath);
    }
}

void StatisticsWindow::exportSelectedSession(int sessionId)
{
    if (!dbManager | !sessionId || sessionId == 0)
        return;

    QList<TestSessionData> data = dbManager->returnExportDataForSession(sessionId);
    //    if (data.isEmpty()) {
    //        return; // Show error - TODO
    //    }

    QString dir = QFileDialog::getExistingDirectory(this, tr("Open Directory"),
                                                    "/home",
                                                    QFileDialog::ShowDirsOnly
                                                        | QFileDialog::DontResolveSymlinks);

    if (!dir.isEmpty()) {
        CSVExporter exporter;
        QString filePath = QFileDialog::getSaveFileName(this, tr("Save CSV"),
                                                        "/home/username/TestingSessions_" + QString::number(sessionId) + ".csv",
                                                        tr("CSV Files (*.csv)"));

        exporter.exportToCSV(data, filePath);
    }
}
