//=======================================================================
/**
 * @file TrainingWindow.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "TrainingWindow.h"

#include <QLabel>
#include <QVBoxLayout>
#include <QListWidget>
#include <QFileDialog>
#include <QFileInfo>
#include <QMessageBox>
#include <QTimer>

#include "AudioFile.h"
#include "FrequencyResponseHelper.h"
#include "FileListWidget.h"
#include "ShelfFilter.h"

TrainingWindow::TrainingWindow(QWidget *parent) : QWidget(parent)
{
    // No op.
}

TrainingWindow::~TrainingWindow()
{
    workerThread->quit();
    workerThread->wait();
    jackWorker->stopJackServer();
}

QWidget *TrainingWindow::createWidget()
{
    QWidget* mainWidget = new QWidget();

    landingWidget = createLandingWidget();
    trainingWidget = createTrainingWidget();

    widgetContainer.addWidget(landingWidget);
    widgetContainer.addWidget(trainingWidget);

    widgetContainer.setCurrentIndex(1);

    // Create a vertical layout for the stacked widget and add it to it
    QVBoxLayout* stackedLayout = new QVBoxLayout;
    stackedLayout->addWidget(&widgetContainer);

    // Create a main layout and add the button layout and stacked layout to it
    QVBoxLayout* mainLayout = new QVBoxLayout(mainWidget);
    mainLayout->addLayout(stackedLayout);

    return mainWidget;
}

QWidget *TrainingWindow::createLandingWidget()
{
    QWidget* landingWidget = new QWidget();

    QLabel* sourceLabel = new QLabel("Select audio source for training");
    sourceLabel->setStyleSheet("color: white; font-size: 20pt");
    sourceLabel->setAlignment(Qt::AlignHCenter);

    QString buttonsStyleSheet = "color:white; font-size: 14pt; font-weight: semi-bold; padding: 16px; border: 0.5px solid #67676A; border-radius: 8px; " ;

    QPushButton* otherSourceButton = new QPushButton("Microphone/JACK Compatible Input");
    QPushButton* localFilesButton = new QPushButton("Import Audio File From Computer (WAV)");

    // Button actions
    QObject::connect(localFilesButton, &QPushButton::clicked, [localFilesButton, this]() {
        QStringList audioFiles = QFileDialog::getOpenFileNames(localFilesButton, "Select Audio", "", "Audio Files (*.wav)");

        int successfulInserts = 0;
        int failedInserts = 0;

        for (const QString& audioFilePath : audioFiles) {
            if (!audioFilePath.isEmpty()) {
                QFileInfo fileInfo(audioFilePath);
                QString fileName = fileInfo.fileName();

                if (!dbManager)
                    continue;

                // 1: Store file paths to DB
                bool fileInsertOp = dbManager->insertAudioFile(audioFilePath, fileName);
                if (fileInsertOp) {
                    successfulInserts++;
                } else {
                    failedInserts++;
                }
            }
        }

        if (audioFiles.size() > 0 && ((successfulInserts + failedInserts) == audioFiles.size())) {
            // 2: Hide the landing widget and show the training widget
            showTrainingWidget();
        }

    });

    // Buttons stack
    QHBoxLayout* buttonHStack = new QHBoxLayout;
    buttonHStack->addWidget(localFilesButton);
    buttonHStack->addSpacing(20);
    buttonHStack->addWidget(otherSourceButton);
    buttonHStack->setAlignment(Qt::AlignHCenter);

    localFilesButton->setStyleSheet(buttonsStyleSheet);
    otherSourceButton->setStyleSheet(buttonsStyleSheet);

    QVBoxLayout* mainContentLayout = new QVBoxLayout();
    mainContentLayout->addWidget(sourceLabel);
    mainContentLayout->addStretch();
    mainContentLayout->addLayout(buttonHStack);
    mainContentLayout->addStretch();

    QVBoxLayout* mainLayout = new QVBoxLayout(landingWidget);
    mainLayout->addStretch();
    mainLayout->addLayout(mainContentLayout);
    mainLayout->addStretch();

    return  landingWidget;
}

QWidget *TrainingWindow::createTrainingWidget()
{
    QWidget* trainingWidget = new QWidget();

    QLabel* statusLabel = new QLabel("Status: Stopped");
    cpuLoadLabel = new QLabel("CPU Load: N/A");
    startStopButton = new QPushButton("Start Training");

    QString buttonsStyleSheet = "color: #FFFFFF; padding: 4px" ;

    QVBoxLayout* buttonLayout = new QVBoxLayout;
    buttonLayout->addStretch();
    buttonLayout->addWidget(statusLabel);
    buttonLayout->addSpacing(8);
    buttonLayout->addWidget(cpuLoadLabel);
    buttonLayout->addSpacing(8);
    buttonLayout->addWidget(startStopButton);
    buttonLayout->addStretch();

    statusLabel->setStyleSheet("color: white; font-size: 12pt");
    cpuLoadLabel->setStyleSheet("color: white; font-size: 12pt");
    startStopButton->setStyleSheet(buttonsStyleSheet);

    cpuLoadTimer = new QTimer(this);
    cpuLoadTimer->setInterval(1000);

    // Connect the timer's timeout signal to call the getCpuLoad method
    connect(cpuLoadTimer, &QTimer::timeout, this, [=](){
        if (jackWorker)
            jackWorker->pingCPULoad();
    });

    // File List Widget
    fileListWidget = new FileListWidget;
    fileListContentWidget = fileListWidget->createWidget();
    QVBoxLayout* fileListLayout = new QVBoxLayout;
    fileListLayout->addSpacing(10);
    fileListLayout->addWidget(fileListContentWidget);
    fileListLayout->addStretch();

    connect(fileListWidget, &FileListWidget::didSelectAudioFile, this, [this](QString filePath) {
        qDebug() << filePath;
        didSelectAudioFile(filePath);
    });

    connect(fileListWidget, &FileListWidget::didPushImportFileButton, this, [this]() {

        if (jackWorker->isAudioEngineRunning()) {
            showErrorDialog("Please stop the training session to import more files");
        } else {
            QStringList audioFiles = QFileDialog::getOpenFileNames(fileListWidget, "Select Audio", "", "Audio Files (*.wav)");

            int successfulInserts = 0;
            int failedInserts = 0;

            for (const QString& audioFilePath : audioFiles) {
                if (!audioFilePath.isEmpty()) {
                    QFileInfo fileInfo(audioFilePath);
                    QString fileName = fileInfo.fileName();

                    if (!dbManager)
                        continue;

                    // 1: Store file paths to DB
                    bool fileInsertOp = dbManager->insertAudioFile(audioFilePath, fileName);
                    if (fileInsertOp) {
                        successfulInserts++;
                    } else {
                        failedInserts++;
                    }
                }
            }

            if (audioFiles.size() > 0 && ((successfulInserts + failedInserts) == audioFiles.size())) {
                populateAudioFiles();
            }
        }
    });

    connect(fileListWidget, &FileListWidget::didPushClearFileButton, this, [this]() {
        if (jackWorker->isAudioEngineRunning()) {
            showErrorDialog("Please stop the training session to clear the imported files.");
        } else {
            if (!dbManager)
                return;
            if (dbManager->clearAudioFiles()) {
                fileListWidget->clearList();
            }
        }
    });

    // Play buttons layout
    QPushButton* referenceButton = new QPushButton("Play Reference");
    QPushButton* modifiedButton = new QPushButton("Play Modified");

    referenceButton->setStyleSheet(buttonsStyleSheet);
    modifiedButton->setStyleSheet(buttonsStyleSheet);

    QVBoxLayout* playButtonsLayout = new QVBoxLayout;
    playButtonsLayout->addStretch();
    playButtonsLayout->addWidget(referenceButton);
    playButtonsLayout->addSpacing(10);
    playButtonsLayout->addWidget(modifiedButton);
    playButtonsLayout->addStretch();

    connect(referenceButton, &QPushButton::clicked, [this]() {
        if (jackWorker->isAudioEngineRunning()) {
            jackWorker->setIsPlayingModified(false);
            jackWorker->setAudioFile(currentAudioFile);
        }
    });

    connect(modifiedButton, &QPushButton::clicked, [this]() {
        if (jackWorker->isAudioEngineRunning()) {
            jackWorker->setIsPlayingModified(true);
            jackWorker->setAudioFile(currentAudioFile);
        }
    });

    // THREAD
    QThread::currentThread()->setObjectName("Main Thread");
    qDebug()<<"From main thread: "<<QThread::currentThread()->objectName();

    workerThread = new QThread();
    jackWorker = new JackWorker();
    jackWorker->moveToThread(workerThread);
    // SLOT
    //    connect( worker, &Worker::error, this, &MyClass::errorString);
    connect( workerThread, &QThread::started, jackWorker, &JackWorker::setupJackServer);
    connect( jackWorker, &JackWorker::finished, workerThread, &QThread::quit);
    connect( jackWorker, &JackWorker::finished, jackWorker, &JackWorker::deleteLater);
    connect( workerThread, &QThread::finished, workerThread, &QThread::deleteLater);

    workerThread->start();

    QObject::connect(startStopButton, &QPushButton::clicked, [this]() {
        if (jackWorker->isAudioEngineRunning()) {
            jackWorker->stopJackServer();
            startStopButton->setText("Start Training");
            sourceSelectionDropdown->setEnabled(true);
            sourceSelectionDropdown->setToolTip("");
            emit didChangeTrainingSessionState(SessionType::Stopped);
            cpuLoadTimer->stop();
            cpuLoadLabel->setText("CPU Load: N/A");
        } else {
            jackWorker->startJackServer();
            jackWorker->setVolumeLevel(volumeSlider->sliderPosition());

            jackWorker->setAudioSource(
                sourceSelectionDropdown->currentIndex() == 0 ? SourceType::Local : SourceType::JackInput
                );

            jackWorker->setActiveFilterType(
                filterSelectionDropdown->currentIndex() == 0 ? FilterType::HighShelf : FilterType::LowShelf
                );

            startStopButton->setText("Stop Training");
            sourceSelectionDropdown->setEnabled(false);
            sourceSelectionDropdown->setToolTip("Please stop the training session to change audio source.");

            emit didChangeTrainingSessionState(SessionType::Running);
            jackWorker->pingCPULoad();
            cpuLoadTimer->start();
        }
    });

    connect(jackWorker, &JackWorker::jackEngineStatus, this, [statusLabel, this](QString status) {
        QString statusFormatted = QString("Status: %1").arg(status);
        statusLabel->setText(statusFormatted);
    });

    connect(jackWorker, &JackWorker::updateCPULoad, this, [this](float load) {
        QString loadFormatted = QString("CPU Load: %1 %").arg(QString::number(load, 'f', 2));
        cpuLoadLabel->setText(loadFormatted);
    });

    // Control Panel Layout
    QLabel* controlPanelTitle = new QLabel("Control Panel");
    controlPanelTitle->setAlignment(Qt::AlignLeading);
    controlPanelTitle->setStyleSheet("color: white; font-size: 12pt; font-weight: bold");
    QFont f = controlPanelTitle->font();
    f.setUnderline(true);
    controlPanelTitle->setFont(f);

    // Source selection
    QLabel* sourceSelectionLabel = new QLabel("Select Source");
    sourceSelectionLabel->setStyleSheet("color: white;");
    sourceSelectionLabel->setAlignment(Qt::AlignLeading);
    sourceSelectionLabel->setFixedHeight(20);

    // Source selection dropdpwn
    sourceSelectionDropdown = new CustomComboBox(this);

    // Add items to the source
    sourceSelectionDropdown->addItem("Local audio files");
    sourceSelectionDropdown->addItem("Microphone/JACK");
    sourceSelectionDropdown->setFixedSize(250, 30);
    sourceSelectionDropdown->setStyleSheet(buttonsStyleSheet);

    connect(sourceSelectionDropdown, &QComboBox::activated, this, [this] {
        qDebug() << "SELECTED" << sourceSelectionDropdown->currentIndex();
        if (sourceSelectionDropdown->currentIndex() == 0) {
            fileListContentWidget->setVisible(true);
            jackWorker->setAudioSource(SourceType::Local);
        } else {
            fileListContentWidget->setVisible(false);
            jackWorker->setAudioSource(SourceType::JackInput);
        }
    });

    sourceSelectionLabel->setBuddy(sourceSelectionDropdown);

    QVBoxLayout* sourceSelectionLayout = new QVBoxLayout;
    sourceSelectionLayout->addWidget(sourceSelectionLabel);
    sourceSelectionLayout->addWidget(sourceSelectionDropdown);

    // Filter selection
    QLabel* filterSelectionLabel = new QLabel("Select Filter");
    filterSelectionLabel->setStyleSheet("color: white;");
    filterSelectionLabel->setAlignment(Qt::AlignLeading);
    filterSelectionLabel->setFixedHeight(20);

    // Filter selection dropdpwn
    filterSelectionDropdown = new CustomComboBox(this);

    // Add items to the dropdown
    filterSelectionDropdown->addItem("High Shelf Filter");
    filterSelectionDropdown->addItem("Low Shelf Filter");
    filterSelectionDropdown->setFixedSize(250, 30);
    filterSelectionDropdown->setStyleSheet(buttonsStyleSheet);

    connect(filterSelectionDropdown, &QComboBox::activated, this, [this] {
        qDebug() << "SELECTED" << filterSelectionDropdown->currentIndex();
        if (filterSelectionDropdown->currentIndex() == 0) {
            didSelectAudioFilter(FilterType::HighShelf);
        } else {
            didSelectAudioFilter(FilterType::LowShelf);
        }
    });

    filterSelectionLabel->setBuddy(filterSelectionDropdown);

    QVBoxLayout* filterSelectionLayout = new QVBoxLayout;
    filterSelectionLayout->addWidget(filterSelectionLabel);
    filterSelectionLayout->addWidget(filterSelectionDropdown);

    // Filter parameters
    QLabel* filterParametersLabel = new QLabel("Filter Parameters");
    filterParametersLabel->setAlignment(Qt::AlignLeading);
    filterParametersLabel->setStyleSheet("color: white; font-size: 12pt; font-weight: bold");
    QFont fLabel = filterParametersLabel->font();
    fLabel.setUnderline(true);
    filterParametersLabel->setFont(fLabel);

    // Volume slider
    volumeSlider = new QSlider(Qt::Horizontal);
    volumeSlider->setMinimum(0);
    volumeSlider->setMaximum(100);
    volumeSlider->setSliderPosition(50);
    QLabel* volumeSliderTitleLabel = new QLabel("Volume");
    volumeSliderTitleLabel->setStyleSheet("color: white;");
    volumeSliderTitleLabel->setAlignment(Qt::AlignLeading);
    QLabel* volumeSliderValueLabel = new QLabel(QString::number(volumeSlider->value()));
    volumeSliderValueLabel->setStyleSheet("color: white;");
    volumeSliderValueLabel->setAlignment(Qt::AlignTrailing);

    // Gain slider
    gainSlider = new QSlider(Qt::Horizontal);
    gainSlider->setMinimum(-20);
    gainSlider->setMaximum(20);
    gainSlider->setTickInterval(2);
    gainSlider->setSliderPosition(12);
    QLabel* gainSliderTitleLabel = new QLabel("Gain (dB)");
    gainSliderTitleLabel->setStyleSheet("color: white;");
    gainSliderTitleLabel->setAlignment(Qt::AlignLeading);
    QLabel* gainSliderValueLabel = new QLabel(QString::number(gainSlider->value()));
    gainSliderValueLabel->setStyleSheet("color: white;");
    gainSliderValueLabel->setAlignment(Qt::AlignTrailing);

    // Cut off frequency slider
    cutOfFrequencySlider = new QSlider(Qt::Horizontal);
    cutOfFrequencySlider->setMinimum(20);
    cutOfFrequencySlider->setTickInterval(1000);
    cutOfFrequencySlider->setMaximum(24100);
    cutOfFrequencySlider->setSliderPosition(10000);
    QLabel* cutOfFrequencySliderTitleLabel = new QLabel("Frequency (Hz)");
    cutOfFrequencySliderTitleLabel->setStyleSheet("color: white;");
    cutOfFrequencySliderTitleLabel->setAlignment(Qt::AlignLeading);
    QLabel* cutOfFrequencySliderValueLabel = new QLabel(QString::number(cutOfFrequencySlider->value()));
    cutOfFrequencySliderValueLabel->setStyleSheet("color: white;");
    cutOfFrequencySliderValueLabel->setAlignment(Qt::AlignTrailing);

    // Connect the valueChanged signal of each slider to a slot that updates the label text
    connect(volumeSlider, &QSlider::sliderReleased, this, [volumeSliderValueLabel, this] {
        jackWorker->setVolumeLevel(volumeSlider->sliderPosition());
    });
    connect(volumeSlider, &QSlider::valueChanged, this, [volumeSliderValueLabel, this](int value) {
        volumeSliderValueLabel->setText(QString::number(value));
    });

    connect(gainSlider, &QSlider::sliderReleased, this, [gainSliderValueLabel, this] {
        sliderValueDidChange(
            gainSlider->sliderPosition(),
            cutOfFrequencySlider->sliderPosition()
            );
    });
    connect(gainSlider, &QSlider::valueChanged, this, [gainSliderValueLabel, this](int value) {
        gainSliderValueLabel->setText(QString::number(value));
    });

    connect(cutOfFrequencySlider, &QSlider::sliderReleased, this, [cutOfFrequencySliderValueLabel, this] {
        sliderValueDidChange(
            gainSlider->sliderPosition(),
            cutOfFrequencySlider->sliderPosition()
            );
    });
    connect(cutOfFrequencySlider, &QSlider::valueChanged, this, [cutOfFrequencySliderValueLabel, this](int value) {
        cutOfFrequencySliderValueLabel->setText(QString::number(value));
    });


    // Volume
    QHBoxLayout* sliderLabelLayout = new QHBoxLayout;
    sliderLabelLayout->addWidget(volumeSliderTitleLabel);
    sliderLabelLayout->addStretch();
    sliderLabelLayout->addWidget(volumeSliderValueLabel);
    volumeSliderTitleLabel->setFixedHeight(20);
    volumeSliderValueLabel->setFixedHeight(20);

    QVBoxLayout* sliderMainLayout = new QVBoxLayout;
    sliderMainLayout->addLayout(sliderLabelLayout);
    sliderMainLayout->addWidget(volumeSlider);
    volumeSlider->setFixedWidth(250);


    // Gain
    QHBoxLayout* sliderLabel1Layout = new QHBoxLayout;
    sliderLabel1Layout->addWidget(gainSliderTitleLabel);
    sliderLabel1Layout->addStretch();
    sliderLabel1Layout->addWidget(gainSliderValueLabel);
    gainSliderTitleLabel->setFixedHeight(20);
    gainSliderValueLabel->setFixedHeight(20);

    QVBoxLayout* sliderMain1Layout = new QVBoxLayout;
    sliderMain1Layout->addLayout(sliderLabel1Layout);
    sliderMain1Layout->addWidget(gainSlider);
    gainSlider->setFixedWidth(250);

    // Frequency
    QHBoxLayout* sliderLabel2Layout = new QHBoxLayout;
    sliderLabel2Layout->addWidget(cutOfFrequencySliderTitleLabel);
    sliderLabel2Layout->addStretch();
    sliderLabel2Layout->addWidget(cutOfFrequencySliderValueLabel);
    cutOfFrequencySliderTitleLabel->setFixedHeight(20);
    cutOfFrequencySliderValueLabel->setFixedHeight(20);

    QVBoxLayout* sliderMain2Layout = new QVBoxLayout;
    sliderMain2Layout->addLayout(sliderLabel2Layout);
    sliderMain2Layout->addWidget(cutOfFrequencySlider);
    cutOfFrequencySlider->setFixedWidth(250);

    QVBoxLayout* controlPanelLayout = new QVBoxLayout;
    controlPanelLayout->addSpacing(20);
    controlPanelLayout->addWidget(controlPanelTitle);
    controlPanelLayout->addSpacing(20);
    controlPanelLayout->addLayout(buttonLayout);
    controlPanelLayout->addSpacing(40);
    controlPanelLayout->addLayout(sourceSelectionLayout);
    controlPanelLayout->addSpacing(40);
    controlPanelLayout->addLayout(filterSelectionLayout);
    controlPanelLayout->addSpacing(40);
    controlPanelLayout->addWidget(filterParametersLabel);
    controlPanelLayout->addSpacing(20);
    controlPanelLayout->addLayout(sliderMainLayout);
    controlPanelLayout->addLayout(sliderMain1Layout);
    controlPanelLayout->addLayout(sliderMain2Layout);
    controlPanelLayout->addSpacing(40);
    controlPanelLayout->addLayout(playButtonsLayout);
    controlPanelLayout->addStretch();


    // Graph Layout
    eqGraph = new EQGraph();
    QVBoxLayout* graphLayout = new QVBoxLayout;
    graphLayout->addWidget(eqGraph);
    graphLayout->addStretch();
    eqGraph->setFixedSize(850, 400);


    // Create a horizontal layout to hold the buttons and sliders
    QHBoxLayout* contentLayout = new QHBoxLayout();
    contentLayout->addSpacing(20);
    contentLayout->addLayout(fileListLayout);
    contentLayout->addSpacing(30);
    contentLayout->addLayout(controlPanelLayout);
    contentLayout->addSpacing(40);
    contentLayout->addLayout(graphLayout);
    contentLayout->addStretch();

    // Vertically center the button layout and slider layout in the content widget
    QVBoxLayout* mainLayout = new QVBoxLayout(trainingWidget);
    mainLayout->addStretch();
    mainLayout->addLayout(contentLayout);
    mainLayout->addStretch();
    mainLayout->setContentsMargins(40, 0, 0, 0);

    // The order is important.
    didSelectAudioFilter(FilterType::HighShelf);
    sliderValueDidChange(
        gainSlider->sliderPosition(),
        cutOfFrequencySlider->sliderPosition()
        );

    return trainingWidget;
}

void TrainingWindow::showTrainingWidget()
{
    widgetContainer.setCurrentIndex(1);
    populateAudioFiles();
}

void TrainingWindow::showLandingWidget()
{
    widgetContainer.setCurrentIndex(0);
}

void TrainingWindow::setDatabaseManager(DatabaseManager *manager)
{
    dbManager = manager;
}

void TrainingWindow::populateAudioFiles()
{
    if (!dbManager)
        return;
    QList<AudioSample> audioFiles = dbManager->getAudioFiles();
    fileListWidget->populateList(audioFiles);

    // Select the first file when populated.
    if (audioFiles.size() > 0)
        didSelectAudioFile(audioFiles[0].path);
}

void TrainingWindow::sliderValueDidChange(int gain, int cutOffFrequency)
{

    emit didChangeSliderValue(gain, cutOffFrequency);

    double sample_rate = 44100.0;
    double gain_db = double(gain);
    double cutoff_frequency = double(cutOffFrequency);

    ShelfFilter ShelfFilter(sample_rate, cutoff_frequency, gain_db, activeFilterType);
    std::vector<double> B = ShelfFilter.get_B();
    std::vector<double> A = ShelfFilter.get_A();

    // Choose the frequency points for the plot (e.g., 20 Hz to 24 kHz)
    std::vector<double> frequencies =  FrequencyResponseHelper().logspace(20, 22050, 1000);

    // Calculate the gain (dB) for each frequency point
    std::vector<double> gain_db_values = FrequencyResponseHelper().calculate_frequency_response(B, A, sample_rate, frequencies);

    eqGraph->plot(
        frequencies,
        gain_db_values,
        20,
        "EQ Graph",
        "Normalized Frequency(Hz)",
        "Gain(dB)"
        );

    jackWorker->setFilterConfig(cutOffFrequency, gain);
}

void TrainingWindow::didSelectAudioFile(QString filePath)
{
    if (!filePath.isEmpty()) {
        AudioFile<float> a;
        bool loadedOK = a.load (filePath.toStdString());

        if (!loadedOK)
            return; // TODO: Show an error here.
        currentAudioFile = a;
        isAudioFileValid = true;
        emit didChangeAudioFile(filePath);
    }
}

void TrainingWindow::didSelectAudioFilter(FilterType filterType)
{
    emit didChangeAudioFilter(filterType);
    activeFilterType = filterType;
    jackWorker->setActiveFilterType(filterType);
    sliderValueDidChange(gainSlider->sliderPosition(), cutOfFrequencySlider->sliderPosition());
}

void TrainingWindow::showErrorDialog(const QString &message)
{
    QMessageBox::critical(nullptr, "Error", message);
}
