//=======================================================================
/**
 * @file Window.cpp
 * @author Priyonto M rahman
 * @copyright Copyright (C) 2023 Priyonto M rahman
 *
 * MIT License
 *
 * Copyright (c) 2023 Priyonto M rahman
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in all
 * copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */
//=======================================================================

#include "Window.h"

#include <QVBoxLayout>
#include <QHBoxLayout>
#include <QDebug>

Window::Window(QWidget *parent) : QWidget(parent) {
    setUpDatabaseConnection();

    // Create the content widgets
    trainingWindow = new TrainingWindow(&contentWidgetContainer);
    testingWindow = new TestingWindow(&contentWidgetContainer);
    statisticsWindow = new StatisticsWindow(&contentWidgetContainer);

    // Add the content widgets to the stacked widget
    contentWidgetContainer.addWidget(trainingWindow->createWidget());
    contentWidgetContainer.addWidget(testingWindow->createWidget());
    contentWidgetContainer.addWidget(statisticsWindow->createWidget());

    trainingWindow->setDatabaseManager(dbManager);
    trainingWindow->populateAudioFiles();

    connect(trainingWindow, &TrainingWindow::didChangeTrainingSessionState, this, [this](auto sessionType) {
        if (sessionType == SessionType::Running) {
            testingWindowButton->setEnabled(false);
            statisticsWindowButton->setEnabled(false);
            testingWindowButton->setToolTip("Testing panel is disabled if there's an active training session in progress.");
            statisticsWindowButton->setToolTip("Statistics panel is disabled if there's an active training session in progress.");
        } else {
            testingWindowButton->setEnabled(true);
            statisticsWindowButton->setEnabled(true);
            testingWindowButton->setToolTip("");
            statisticsWindowButton->setToolTip("");
        }
    });

    connect(trainingWindow, &TrainingWindow::didChangeSliderValue, this, [this](int gain, int cutOffFrequency) {
        testingWindow->updateFilterParameters(gain, cutOffFrequency);
    });

    connect(trainingWindow, &TrainingWindow::didChangeAudioFile, this, [this](auto filePath) {
        testingWindow->updateSelectedAudioFile(filePath);
    });

    connect(trainingWindow, &TrainingWindow::didChangeAudioFilter, this, [this](auto filter) {
        testingWindow->updateSelectedFilterType(filter);
    });

    testingWindow->setDatabaseManager(dbManager);
    statisticsWindow->setDatabaseManager(dbManager);

    // Create the buttons to switch between the content widgets
    trainingWindowButton = new QPushButton("Training Mode");
    testingWindowButton = new QPushButton("Test Mode");
    statisticsWindowButton = new QPushButton("Statistics");

    trainingWindowButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    trainingWindowButton->setFixedWidth(buttonWidth);
    testingWindowButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    testingWindowButton->setFixedWidth(buttonWidth);
    statisticsWindowButton->setSizePolicy(QSizePolicy::Fixed, QSizePolicy::Fixed);
    statisticsWindowButton->setFixedWidth(buttonWidth);

    updateButtonStyles(trainingWindowButton);

    connect(trainingWindowButton, &QPushButton::clicked, this, &Window::showTrainingWindow);
    connect(testingWindowButton, &QPushButton::clicked, this, &Window::showTestingWindow);
    connect(statisticsWindowButton, &QPushButton::clicked, this, &Window::showStatisticsWindow);

    // Create a horizontal layout for the buttons and add them to it
    QHBoxLayout* buttonLayout = new QHBoxLayout;

    // Create a spacer item to keep empty space to the right of the buttons
    QSpacerItem* leadingSpacerItem = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
    buttonLayout->addItem(leadingSpacerItem);

    // Add buttons
    buttonLayout->addWidget(trainingWindowButton);
    buttonLayout->addWidget(testingWindowButton);
    buttonLayout->addWidget(statisticsWindowButton);
    buttonLayout->setSpacing(0);

    // Create a spacer item to keep empty space to the right of the buttons
    QSpacerItem* trailingSpacerItem = new QSpacerItem(0, 0, QSizePolicy::Expanding, QSizePolicy::Minimum);
    // Add the spacer item to the end of the button layout
    buttonLayout->addItem(trailingSpacerItem);

    // Create a vertical layout for the stacked widget and add it to it
    QVBoxLayout* stackedLayout = new QVBoxLayout;
    stackedLayout->addWidget(&contentWidgetContainer);

    // Create a main layout and add the button layout and stacked layout to it
    QVBoxLayout* mainLayout = new QVBoxLayout(this);
    mainLayout->addLayout(buttonLayout);
    mainLayout->addLayout(stackedLayout);
    mainLayout->setContentsMargins(0, 0, 0, 0);

    // Set the background color of the main layout
    this->setStyleSheet("background-color: #383961;");

    resize(1400, 500);
}

Window::~Window() {
    delete dbManager;
}

void Window::setUpDatabaseConnection() {
    dbManager = new DatabaseManager;
    if (!dbManager->setupDatabase()) {
        qDebug() << "Failed to set up the database.";
    }
}

void Window::showTrainingWindow() {
    contentWidgetContainer.setCurrentIndex(0);
    updateButtonStyles(trainingWindowButton);
}

void Window::showTestingWindow() {
    contentWidgetContainer.setCurrentIndex(1);
    testingWindow->populateAudioFiles();
    updateButtonStyles(testingWindowButton);
}

void Window::showStatisticsWindow() {
    contentWidgetContainer.setCurrentIndex(2);
    statisticsWindow->populateSessions();
    updateButtonStyles(statisticsWindowButton);
}

void Window::updateButtonStyles(QPushButton* selectedButton) {
    trainingWindowButton->setStyleSheet(notSelectedStyleSheet);
    testingWindowButton->setStyleSheet(notSelectedStyleSheet);
    statisticsWindowButton->setStyleSheet(notSelectedStyleSheet);

    selectedButton->setStyleSheet(selectedStyleSheet);
}
