#include "TestDatabaseManager.h"

#include <QtTest>

void TestDatabaseManager::initTestCase() {
    dbManager = new DatabaseManager();
}

void TestDatabaseManager::cleanupTestCase() {
    delete dbManager;
}

void TestDatabaseManager::testSetupDatabase() {
    QVERIFY(dbManager->setupDatabase());
}

void TestDatabaseManager::testDropAndRecreateTables() {
    dbManager->dropAndRecreateTables();
    // After dropping and recreating tables, there should be no audio files
    QVERIFY(dbManager->getAudioFiles().isEmpty());
}

void TestDatabaseManager::testInsertAudioFile() {
    QVERIFY(dbManager->insertAudioFile("/path/to/file", "file"));
    // After inserting, there should be at least one audio file
    QVERIFY(!dbManager->getAudioFiles().isEmpty());
}

void TestDatabaseManager::testGetAudioFiles() {
    dbManager->insertAudioFile("/path/to/file", "file");
    QList<AudioSample> files = dbManager->getAudioFiles();
    QVERIFY(!files.isEmpty());
    // Assuming you can access file path in AudioSample
    QCOMPARE(files.first().path, QString("/path/to/file"));
}

void TestDatabaseManager::testClearAudioFiles() {
    dbManager->insertAudioFile("/path/to/file", "file");
    QVERIFY(dbManager->clearAudioFiles());
    // After clearing, there should be no audio files
    QVERIFY(dbManager->getAudioFiles().isEmpty());
}

void TestDatabaseManager::testCreateTestSession() {
    int sessionId = dbManager->createTestSession(1, 10);
    QVERIFY(sessionId != -1);
    // Add more checks here based on what you expect after creating a session
}

void TestDatabaseManager::testUpdateTestSessionEndTime() {
    int sessionId = dbManager->createTestSession(1, 10);
    QVERIFY(dbManager->updateTestSessionEndTime(sessionId));
    // Add more checks here based on what you expect after updating a session
}

void TestDatabaseManager::testGetSessionsWithDuration() {
    int sessionId = dbManager->createTestSession(1, 10);
    QVector<TestingSession> sessions = dbManager->getSessionsWithDuration();
    QVERIFY(!sessions.isEmpty());
    // Add more checks here based on what you expect after getting sessions with duration
}

void TestDatabaseManager::testInsertTrial() {
    int sessionId = dbManager->createTestSession(1, 10);
    QVERIFY(dbManager->insertTrial(sessionId, "correctResponse", "userResponse", 10));
    // After inserting a trial, there should be at least one trial for this session
    QVERIFY(!dbManager->getTrialsForSession(sessionId).isEmpty());
}

void TestDatabaseManager::testGetTrialsForSession() {
    int sessionId = dbManager->createTestSession(1, 10);
    dbManager->insertTrial(sessionId, "correctResponse", "userResponse", 10);
    QVector<SessionTrial> trials = dbManager->getTrialsForSession(sessionId);
    QVERIFY(!trials.isEmpty());
    // Assuming you can access responseTime in SessionTrial
    QCOMPARE(trials.first().responseTime, 10);
}

void TestDatabaseManager::testGetAverageTrialsPerSession() {
    // Clear the database before the test
    dbManager->dropAndRecreateTables();

    // Assuming initial state with no sessions or trials
    QVERIFY(qFuzzyIsNull(dbManager->getAverageTrialsPerSession()));

    // After adding one session with one trial
    int sessionId = dbManager->createTestSession(1, 1);
    dbManager->insertTrial(sessionId, "correctResponse", "userResponse", 10);
    QCOMPARE(dbManager->getAverageTrialsPerSession(), 1.0);
}

void TestDatabaseManager::testGetAverageResponseTimePerTrial() {
    // Clear the database before the test
    dbManager->dropAndRecreateTables();
    // Assuming initial state with no trials
    QVERIFY(qFuzzyIsNull(dbManager->getAverageResponseTimePerTrial()));
    // After adding one session with one trial
    int sessionId = dbManager->createTestSession(1, 1);
    dbManager->insertTrial(sessionId, "correctResponse", "userResponse", 10);
    QCOMPARE(dbManager->getAverageResponseTimePerTrial(), 10/1000);
}

void TestDatabaseManager::testGetAverageDurationPerSession() {
    // Clear the database before the test
    dbManager->dropAndRecreateTables();
    // Assuming initial state with no sessions
    QVERIFY(qFuzzyIsNull(dbManager->getAverageDurationPerSession()));
    // After adding one session
    int sessionId = dbManager->createTestSession(1, 1);
    // Assuming that the duration of a session is somehow calculated and updated
    // For this test, the actual calculation method doesn't matter
    dbManager->updateTestSessionEndTime(sessionId);
    QVERIFY(dbManager->getAverageDurationPerSession() >= 0);
}

void TestDatabaseManager::testGetAverageSuccessPerSession() {
    // Clear the database before the test
    dbManager->dropAndRecreateTables();
    // Assuming initial state with no trials
    QVERIFY(qFuzzyIsNull(dbManager->getAverageSuccessPerSession()));
    // After adding one session with one successful trial
    int sessionId = dbManager->createTestSession(1, 1);
    dbManager->insertTrial(sessionId, "correctResponse", "correctResponse", 10);
    QCOMPARE(dbManager->getAverageSuccessPerSession(), 100);
}

#include "TestDatabaseManager.moc"
